%
% This is a script to handle the raman function in the optimization
% routine. (lsqcurvefit)
%
% Author Eric Shorr, July 2006 - May 2007
% invoke the optimization routine by merely typing 
% [e0,gamma,u, sig, delta, alife, err] =
% runRaman(1,'ramabs.in',absorp_file,raman_profile,outputFile)
% To have it merely run the ramabs function once, type
% [e0,gamma,u, sig, delta, alife, err] =
% runRaman(0,'ramabs.in',absorp_file,raman_profile,outputFile)
%

function [e0,gamma,u, sig, delta, alife, err] = runRaman(choice, ifile1, ifile2,ifile3, outputFile)
    % datafile1 = absorbtion spectrum
    % datafile2 = raman profiles
    % in order for this program/function to work properly,
    % inram.mexw32, ramfunc.mexw32, and ramabs.in must be in the same directory
    % use ramabs.in to change the input variables and look at inram.for 
    % in case the formatting is unknown    
    
    % load in all variables
    [nmode, nline, ntime, nstep, sig, temp, e0,gamma, rkappa,...
        u, alow, ahigh, delt, refrac,wg, delta,nquanta, lb, ub, ...
        abs_weight, raman_weight, alife] = inram(ifile1);
    % load -mat testRaman.m
         
    
    %
    % datafile2 is a structure containing both characters and numbers so
    % therefore must be explicitly accessed through the .data extenstion.
    % 
    
    datafile1 = importdata(ifile2);
    datafile2 = importdata(ifile3); 
    
    % checking if output file is defined
    
    if( ischar(outputFile(1:length(outputFile))) ) 
        % check if it is a txt file
        if( length(outputFile) < 4 )
            error('output file must be at least 5 characters long');
        end
        suffix = outputFile( (length(outputFile) - 3):length(outputFile) );
        suffix = lower(suffix);
        if( ~strcmp(suffix,'.txt') )
            error('output file must be a .txt');
        end
    end    
    
    
    %
    % xdata = the experimental data points plotted versus ydata. 
    % it also the empirical xfreq points
    % xdata1 = experimental xfreq in the absorption spectrum
    % xdata2 = experimantal xfreq in the raman profiles
    %
    
    xdata1 = datafile1(:,1);
    xdata2 = zeros(size(datafile2,1),nline);
    xdata2(1:size(datafile2,1),1) = datafile2(:,1); 
    
    % creating a duplicate set of x points for each set y points
    for i = 2:nline     
        xdata2(:,i) = xdata2(:,1);
    end
    
    %
    % ydata = data from both input files (Y)
    % ydata1 = absorbtion spectrum (xs)
    % ydata2 = raman profiles (spectrum)
    %
    
    ydata1 = abs_weight*datafile1(:,2);
    ydata2 = raman_weight*datafile2(:,2:(nline + 1));
       
    % routine to find out which, if any sectors are not a number
    len = size(ydata2,1)*ones(nline,1);
    for j = 1:nline % using unsorted list deletion algorithm
        i = 1;
        while( i <= len(j) ) 
            if( isnan(ydata2(i,j)) ) % delete both elements in xdata and ydata
                
                if( i ~= len(j) ) % check if the last element is blank
                    for k = i:(len(j) - 1)
                        xdata2(k,j) = xdata2(k + 1,j); % move everything down one
                        ydata2(k,j) = ydata2(k + 1,j);
                    end
                end
                                
                xdata2(len(j),j) = 0; % just being anal
                ydata2(len(j),j) = 0;
                len(j) = len(j) - 1; % don't allow for the element to be seen again                                
                
            end                       
            if( isnan(ydata2(i,j)) ) % checking if the element before the one deleted is not a number
                % nothing
            else
                i = i + 1;
            end
        end
    end

    xdata = zeros(size(xdata1,1),1 + nline);
    xdata(1:size(xdata1,1),1) = xdata1;
    xdata(1:size(xdata2,1),2:(nline + 1)) = xdata2;

    
    ydata = zeros(size(ydata1,1),nline + 1);    
    ydata(1:size(ydata1,1),1) = ydata1;
    ydata(1:size(ydata2,1),2:(nline + 1)) = ydata2;
    
   
    % e0 = [1]; gamma = [2]; u = [3]; sig = [4]; alife = [5]; delta[6:35]
    
    %
    % x0 holds all parameters of guesses
    % guesses are being divided so that they are the same units when they
    % are changed.
    %
    x0 = zeros(3 + nmode,1);
    x0(1) = e0/10000;
    x0(2) = gamma/1000;
    x0(3) = u;
    x0(4) = sig/1000;
    x0(5) = alife/100;
    x0(6:(5 + nmode)) = delta(1:nmode);
    
    lb(1) = lb(1)/10000; % e0 lower bounds / 10000
    lb(2) = lb(2)/1000;  % gamma lower bounds / 1000
    lb(4) = lb(4)/1000;  % sig lower bounds / 1000
    lb(5) = lb(5)/100;   % alife lower bounds / 100
    % u [ lb(3) ] and delta lb(5:(nmode + 4)) don't need to be changed
    
    ub(1) = ub(1)/10000; % e0 upper bounds / 10000
    ub(2) = ub(2)/1000;  % gamma upper bounds / 1000
    ub(4) = ub(4)/1000;  % sig upper bounds / 1000
    ub(5) = ub(5)/100;   % alife upper bounds / 100
    % u [ ub(3) ] and delta ub(5:(nmode + 4)) don't need to be changed

    % globals for ramabs function
    count = 1; % to print out function evaluation number
    % initialize the output variables in the global environment so that 
    % they can be plotted later.
    spectrum = 0;
    xs = 0;
    xfreq = 0; 
    cdata = 0;
    
    warninglb = zeros(length(x0),1);
    warningub = zeros(length(x0),1);

    %
    %   Processing
    %
    if( choice == 0 ) % just do one run instead
        tic
        [spectrum,xs,xfreq] = ramfunc(int32(nmode), int32(nline),...
        int32(ntime), int32(nstep), sig, temp, e0,gamma, rkappa, u,...
        alow, ahigh, delt, refrac,wg, delta,int32(nquanta),alife);            
        err = 0;

        toc
    else
        if( alow > xdata1(1) )
            error('Out of bounds. Lower limit(alow) needs to be lower');
        else
            if( ahigh < xdata1(length(xdata1)))
                error('Out of bounds. Upper limit(ahigh) needs to be higher');
            else
                if( choice == 1 )
                     options = optimset('LargeScale','on','Display','iter',...
                    'TolX',1e-6,'TolFun',1e-6,'LevenbergMarquardt','off',...
                    'DiffMinChange',1e-2,'DiffMaxChange',0.1,...
                    'MaxFunEvals',100*length(x0));
                   tic % start timer
                   [x,resnorm] = lsqcurvefit(@ramabs, x0, xdata, ydata, lb, ub, options); % fit for all
                   toc % print time elapsed
                   
                   % checking if any of the variables went out of bounds
                                     
                   
                   for i = 1:length(x0)
                       if( warninglb(i) == 1 )
                           % state that this variable went below the lower
                           % bounds
                           if( i == 1 )
                               warning('e0 went below the lower bounds');
                           end
                           if( i == 2 ) 
                               warning('gamma went below the lower bounds');
                           end
                           if( i == 3 )
                               warning('u went below the lower bounds');
                           end
                           if( i == 4 )
                               warning('sig went below the lower bounds');
                           end
                           if( i == 5 )
                               warning('alife went below the lower bounds');
                           end
                           if( i >= 6 )
                               warning('delta "%s" went below the lower bounds', num2str(i - 5 ));
                           end
                       else
                           if( warningub(i) == 1 )
                               % state that this variable went above the
                               % upper bounds
                               if( i == 1 )
                                   warning('e0 went above the upper bounds');
                               end
                               if( i == 2 ) 
                                   warning('gamma went above the upper bounds');
                               end
                               if( i == 3 )
                                   warning('u went above the upper bounds');
                               end
                               if( i == 4 )
                                   warning('sig went above the upper bounds');
                               end
                               if( i == 5 )
                                   warning('alife went above the upper bounds');
                               end
                               if( i >= 6 )
                                   warning('delta "%s" went above the upper bounds', num2str(i - 5 ));
                               end
                           end
                       end
                   end
                                                      
                
                    % calculating the percent error (correlation)
                    % resnorm = 1/2*sum((ramabs(x0,ydata) - ydata)^2) 
                    % rsquared = variance of explained over variance over
                    % observed
                    % err = all the rsquared's times each other = correlation                  
                    
                    mean = sum(sum(ydata)) / ( size(ydata,1) + sum(len));
                    % calculate observed 
                    variance = 0;
                    for i = 1:size(ydata,2)
                        j = 1;
                        while( ydata(j,i) > 0 && j < size(ydata,1) )
                            variance = variance + ( ydata(j,i) - mean ) ^ 2;
                            j = j + 1;
                        end
                    end
                    
                    err = sqrt(2*resnorm/variance);
                                                         
                end
                                
                x(1) = x(1)*10000; % e0
                x(2) = x(2)*1000;  % gamma
                x(4) = x(4)*1000;  % sig
                x(5) = x(5)*100;   % alife
                
                e0 = x(1); 
                gamma = x(2); 
                u = x(3);
                sig = x(4);
                alife = x(5);
                delta(1:nmode) = x(6:(nmode+5));
                          
                          
                % printing quick and dirty for now
                out = zeros(size(spectrum,1),size(spectrum,2)*4);
                out(1:length(xfreq),1) = xfreq(:);
                out(1:length(xs),2) = xs(:);
                out(1:length(xdata),3) = xdata(1:size(xdata,1),1);
                out(1:size(ydata,1),4) = ydata(1:size(ydata,1),1);
                
                for i = 1:nline
                    out(1:length(xfreq),1 + 4*i) = xfreq(:);
                    out(1:length(xfreq),2 + 4*i) = spectrum(1:length(xfreq),i);
                    out(1:len(i),3 + 4*i) = xdata(1:len(i),1 + i);
                    out(1:len(i),4 + 4*i) = ydata(1:len(i),1 + i);
                end
                
                % writing whole matrix to output file
                 
                dlmwrite(outputFile,out,' '); 
                
                % end output to file                                               

            end
        end
    end
    
    
    
    % output
    % visually checking to see if it looks right
    if( ahigh > xdata2(size(xdata2,1)) || alow < xdata2(1) ) % don't want to graph the error cases
        figure(100);
        plot(xfreq(1:size(xfreq,1),1),xs(1:size(xs,1),1),'b',...
        datafile1(:,1),datafile1(:,2),'r');
        legend('Calculated','Experimental',2);
        title('Absorption Spectrum','FontWeight','bold');
        axis([alow ahigh 0 Inf]);
        grid on;
        
        count_figs = 1; % keep track of all the figures/windows drawn for the raman profiles
        for i = 1:nline     % loop to graph raman profiles
            if( mod(i,4) == 1 )     % if the next element is greater than a multiple of 4
                figure(count_figs); % create a new window
                count_figs = count_figs + 1;
                j = 1;              % reset back to the beginning
            end
            subplot(2,2,j);
            j = j + 1;              % go to next spot on window
            plot(xfreq(1:size(xfreq,1)),spectrum(1:size(xfreq,1),i),'b',...
            xdata2(1:len(i),i),ydata2(1:len(i),i),'+r');
            prof = strcat('Raman Profile-',num2str(i));
            title(prof);
            axis([alow ahigh 0 Inf]);
            grid on;
        end
    end
     
    
    % simplifying raman function to only 5 needed parameters
    % F needs to be 1001x40 
        function F = ramabs(x0, xdata)
       % keep track of all function evaluations
            count 
       % printing off all the variables by having no semicolons 

            e0 = x0(1)*10000       
            gamma = x0(2)*1000        
            u = x0(3)
            sig = x0(4)*1000
            alife = x0(5)*100
            delta = zeros(nmode,1);
            delta = x0(6:(nmode + 5)) 
            
            % warning if any variables go out of bounds
            % cycle through variables and check against each upper and
            % lower bounds
            
            for i = 1:length(x0)
                if( x0(i) < lb(i) )
                    warninglb(i) = 1;
                else
                    if( x0(i) > ub(i) )
                        warningub(i) = 1;
                    end
                end
            end

            % if e0 (and all variables) are NaN, exit
            if( isnan(e0) ) 
                return; 
            end
            if( isnumeric(e0) )

                [spectrum,xs,xfreq] = ramfunc(int32(nmode), int32(nline),...
                int32(ntime), int32(nstep), sig, temp, e0,gamma, rkappa, u,...
                alow, ahigh, delt, refrac,wg, delta,int32(nquanta),alife);

                count = count + 1;
                cdata = zeros(size(xdata,1),nline + 1); % calculated data holder which will later be outputted
                
                singxdata = xdata(:,1); % using only the first set of points
                % matching all of the experimental points to the calculated
                % points. xfreq being the calculated X, xs being the
                % calculated Y, and singxdata being the experimental X.
                % Please note that none of these vectors can be 2
                % dimensional therefore a conversion was needed.
                rxs = interp1(xfreq,xs,singxdata); 
                % putting weight on for absorption spectrum
                rxs = abs_weight*rxs;

                cdata(1:size(rxs,1),1) = rxs(1:size(rxs,1));
                
                for profile = 2:(nline + 1) % number of raman profiles + 1
                    % transforming 2 dimensional vectors into 1 dimensional
                    % vectors

                   singxdata = zeros(len(profile - 1),1); % using the second set of points
                   singxdata(:) = xdata(1:len(profile - 1),profile);
                   singspectrum = zeros(nstep,1);
                   singspectrum(1:nstep) = spectrum(1:nstep,(profile - 1));                   

                   rspectrum = interp1(xfreq,singspectrum,singxdata);
                   % putting weight on for raman profiles
                   rspectrum = raman_weight*rspectrum;

                   cdata(1:size(rspectrum,1),profile) = rspectrum(1:size(rspectrum,1));
                end 
                toc            % output total time so far
                F = cdata;     % return calculated data;
                
            end            
        end       
end
    



